import { Directive, ElementRef, forwardRef, OnDestroy, OnInit } from '@angular/core';
import { ControlValueAccessor, NG_VALUE_ACCESSOR } from '@angular/forms';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { MentionsComponent } from './mentions.component';
/**
 * The NgxMentionsAccessorDirective directive is used to indicate the input element.
 * This directive is required.
 */
var /**
 * The NgxMentionsAccessorDirective directive is used to indicate the input element.
 * This directive is required.
 */
MentionsAccessorDirective = /** @class */ (function () {
    function MentionsAccessorDirective(element, host) {
        this.element = element;
        this.host = host;
    }
    MentionsAccessorDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.host.valueChanges.pipe(takeUntil(this._destroyed)).subscribe(function (value) { return _this.onChange(value); });
    };
    MentionsAccessorDirective.prototype.ngOnDestroy = function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    MentionsAccessorDirective.prototype.registerOnChange = function (fn) {
        this._onChange = fn;
    };
    MentionsAccessorDirective.prototype.registerOnTouched = function (fn) {
        this._onTouch = fn;
    };
    MentionsAccessorDirective.prototype.setDisabledState = function (isDisabled) {
        this.host.disabled = isDisabled;
    };
    MentionsAccessorDirective.prototype.writeValue = function (value) {
        if (typeof value === 'string' || value === null) {
            this.host.value = value;
        }
    };
    MentionsAccessorDirective.prototype.onChange = function (value) {
        if (this._onChange) {
            this._onChange(value);
        }
    };
    MentionsAccessorDirective.prototype.onTouched = function () {
        if (this._onTouch) {
            this._onTouch();
        }
    };
    return MentionsAccessorDirective;
}());
/**
 * The NgxMentionsAccessorDirective directive is used to indicate the input element.
 * This directive is required.
 */
export { MentionsAccessorDirective };
//# sourceMappingURL=mentions-input.directive.js.map