import { AfterViewChecked, AfterViewInit, ChangeDetectorRef, ComponentFactoryResolver, ElementRef, EventEmitter, NgZone, OnChanges, OnDestroy, OnInit, SimpleChanges, TemplateRef, ViewContainerRef } from '@angular/core';
import { Subject } from 'rxjs';
import { MentionsListComponent } from './mentions-list.component';
export interface Tag {
    indices: {
        start: number;
        end: number;
    };
}
export interface Line {
    originalContent: string;
    content: string;
    parts: Array<string | Mention>;
}
export interface Mention {
    contents: string;
    tag: Tag;
}
export declare class HighlightedDirective {
    tag: Tag;
}
/**
 *
 */
export declare class MentionsComponent implements OnChanges, OnInit, AfterViewInit, AfterViewChecked, OnDestroy {
    private element;
    private componentResolver;
    private viewContainer;
    private changeDetector;
    private ngZone;
    /**
     * The character to trigger the mentions list when a user is typing in the input field
     */
    triggerChar: string;
    /**
     * The markup used to format a mention in the model value
     */
    mentionMarkup: string;
    /**
     * Optional. When using a custom search (i.e. an API call), the internal searching capability should be disabled.
     */
    disableSearch: boolean;
    /**
     * Only used when internal search is not disabled. This limits the maximum number of items to display in the search
     * result list.
     */
    maxItems: number;
    /**
     * Used to cause the search result list to display in a "drop up" fashion, instead of a typical dropdown.
     */
    dropUp: boolean;
    /**
     * If the supplied mentions are a list of objects, this is the name of the property used to display
     * the mention in the search result list and when formatting a mention in the displayed text.
     */
    displayName: string;
    /**
     * Classes for textarea
     */
    formClass: string;
    placeholder: string;
    value: string;
    required: boolean;
    disabled: boolean;
    rows: number | string;
    columns: number | string;
    /**
     * The list of mentions to display, or filter, in the search result list.
     */
    mentionItems: any[];
    /**
     * An event emitted, after the trigger character has been typed, with the user-entered search string.
     */
    readonly search: EventEmitter<string>;
    readonly valueChanges: EventEmitter<string>;
    readonly stateChanges: Subject<void>;
    mentionListTemplate: TemplateRef<any>;
    textAreaInputElement: ElementRef;
    highlighterElement: ElementRef;
    displayContent: string;
    lines: Line[];
    highlighterStyle: {
        [key: string]: string;
    };
    textAreaClassNames: {
        [key: string]: boolean;
    };
    selectionStart: number;
    selectionEnd: number;
    mentions: any[];
    readonly readonly: string;
    readonly errorState: boolean;
    private _value;
    private _required;
    private _disabled;
    private _rows;
    private _columns;
    private searchString;
    private startPos;
    private startNode;
    mentionsList: MentionsListComponent;
    private stopSearch;
    private markupSearch;
    private _destroyed;
    private newLine;
    private _errorState;
    constructor(element: ElementRef, componentResolver: ComponentFactoryResolver, viewContainer: ViewContainerRef, changeDetector: ChangeDetectorRef, ngZone: NgZone);
    ngOnInit(): void;
    ngOnChanges(changes: SimpleChanges): void;
    ngAfterViewInit(): void;
    ngAfterViewChecked(): void;
    ngOnDestroy(): void;
    onWindowResize(): void;
    onTextAreaScroll(): void;
    onSelect(event: any): void;
    onChange(newPlainTextValue: string): void;
    onKeyDown(event: any): void;
    onBlur(event: MouseEvent | KeyboardEvent): void;
    isPartMention(part: any): boolean;
    formatMention(mention: Mention): string;
    private displayTransform(..._);
    private _formatMention(contents);
    private stopEvent(event);
    private setupMentionsList(caretPosition);
    private handleKeyDown(event, caretPosition, characterPressed);
    private getDisplayValue(item);
    private showMentionsList();
    private updateMentionsList();
    private parseMarkup();
    private parseLines(value?);
    private formatMentions(line);
    private refreshStyles();
    private triggerChange(value);
}
