import { AfterViewChecked, AfterViewInit, ChangeDetectorRef, Component, ComponentFactoryResolver, ContentChild, Directive, ElementRef, EventEmitter, HostListener, Input, NgZone, OnChanges, OnDestroy, OnInit, Output, SimpleChanges, TemplateRef, ViewChild, ViewContainerRef, ViewEncapsulation } from '@angular/core';
import { Subject } from 'rxjs';
import { Key } from './key';
import { MentionsListComponent } from './mentions-list.component';
import { applyChangeToValue, escapeRegExp, findStartOfMentionInPlainText, getBoundsOfMentionAtPosition, getCaretPosition, mapPlainTextIndex, MarkupMention, markupToRegExp, replacePlaceholders, setCaretPosition, styleProperties } from './utils';
var HighlightedDirective = /** @class */ (function () {
    function HighlightedDirective() {
    }
    return HighlightedDirective;
}());
export { HighlightedDirective };
/**
 *
 */
var /**
 *
 */
MentionsComponent = /** @class */ (function () {
    function MentionsComponent(element, componentResolver, viewContainer, changeDetector, ngZone) {
        this.element = element;
        this.componentResolver = componentResolver;
        this.viewContainer = viewContainer;
        this.changeDetector = changeDetector;
        this.ngZone = ngZone;
    }
    Object.defineProperty(MentionsComponent.prototype, "formClass", {
        get: /**
           * Classes for textarea
           */
        function () {
            return Object.keys(this.textAreaClassNames).join(' ');
        },
        set: function (classNames) {
            var _this = this;
            this.textAreaClassNames = {};
            Array.from(classNames.split(' ')).forEach(function (className) {
                _this.textAreaClassNames[className] = true;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this.parseLines(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (value) {
            this._disabled = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "rows", {
        get: function () {
            return this._rows;
        },
        set: function (value) {
            this._rows = Math.max(1, value);
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "columns", {
        get: function () {
            return this._columns;
        },
        set: function (value) {
            this._columns = Math.max(1, value);
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "mentionItems", {
        set: /**
           * The list of mentions to display, or filter, in the search result list.
           */
        function (value) {
            this.mentions = value;
            if (this.disableSearch && this.mentionsList) {
                this.mentionsList.items = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "readonly", {
        get: function () {
            return this.disabled ? 'readonly' : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MentionsComponent.prototype, "errorState", {
        get: function () {
            return this._errorState;
        },
        enumerable: true,
        configurable: true
    });
    MentionsComponent.prototype.ngOnInit = function () {
        this.parseMarkup();
    };
    MentionsComponent.prototype.ngOnChanges = function (changes) {
        if ('markup' in changes) {
            this.parseMarkup();
        }
    };
    MentionsComponent.prototype.ngAfterViewInit = function () {
        this.parseLines(this._value);
        this.refreshStyles();
    };
    MentionsComponent.prototype.ngAfterViewChecked = function () {
        this.refreshStyles();
    };
    MentionsComponent.prototype.ngOnDestroy = function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    MentionsComponent.prototype.onWindowResize = function () {
        this.refreshStyles();
    };
    MentionsComponent.prototype.onTextAreaScroll = function () {
        this.highlighterElement.nativeElement.scrollTop = this.textAreaInputElement.nativeElement.scrollTop;
    };
    MentionsComponent.prototype.onSelect = function (event) {
        this.selectionStart = event.target.selectionStart;
        this.selectionEnd = event.target.selectionEnd;
    };
    MentionsComponent.prototype.onChange = function (newPlainTextValue) {
        var _this = this;
        var value = this._value;
        var displayTransform = this.displayTransform.bind(this);
        var selectionStart = this.textAreaInputElement.nativeElement.selectionStart;
        var selectionEnd = this.textAreaInputElement.nativeElement.selectionEnd;
        var bounds = getBoundsOfMentionAtPosition(newPlainTextValue, this.markupSearch, selectionStart, displayTransform);
        if (bounds.start !== -1) {
            newPlainTextValue = newPlainTextValue.substring(0, bounds.start) + newPlainTextValue.substring(bounds.end);
        }
        var newValue = applyChangeToValue(value, this.markupSearch, newPlainTextValue, this.selectionStart, this.selectionEnd, selectionEnd, displayTransform);
        var startOfMention = findStartOfMentionInPlainText(value, this.markupSearch, selectionStart, displayTransform);
        if (startOfMention.start > -1 && this.selectionEnd > startOfMention.start) {
            selectionStart = startOfMention.start;
            selectionEnd = selectionStart;
        }
        this.selectionStart = Math.max(selectionStart, 0);
        this.selectionEnd = Math.max(selectionEnd, 0);
        this.parseLines(newValue);
        if (this.selectionEnd > 0) {
            setTimeout(function () { return setCaretPosition(_this.textAreaInputElement.nativeElement, _this.selectionEnd); });
        }
    };
    MentionsComponent.prototype.onKeyDown = function (event) {
        var caretPosition = getCaretPosition(this.textAreaInputElement.nativeElement);
        var characterPressed = event.key;
        var keyCode = event.which || event.keyCode;
        if (!characterPressed) {
            var characterCode = event.which || event.keyCode;
            characterPressed = String.fromCharCode(characterCode);
            if (!event.shiftKey && (characterCode >= 65 && characterCode <= 90)) {
                characterPressed = String.fromCharCode(characterCode + 32);
            }
        }
        if (keyCode === Key.Enter && event.wasSelection && caretPosition < this.startPos) {
            caretPosition = this.startNode.length;
            setCaretPosition(this.startNode, caretPosition);
        }
        var startOfMention = findStartOfMentionInPlainText(this._value, this.markupSearch, caretPosition, this.displayTransform.bind(this));
        if (characterPressed === this.triggerChar) {
            this.setupMentionsList(caretPosition);
        }
        else if (startOfMention.start === -1 && this.startPos >= 0) {
            if (caretPosition <= this.startPos) {
                this.mentionsList.show = false;
                this.startPos = -1;
            }
            else if (keyCode !== Key.Shift && !event.shiftKey && !event.metaKey && !event.altKey && !event.ctrlKey &&
                caretPosition > this.startPos) {
                this.handleKeyDown(event, caretPosition, characterPressed);
            }
        }
        else {
            this.onSelect({ target: this.textAreaInputElement.nativeElement });
        }
    };
    MentionsComponent.prototype.onBlur = function (event) {
        if (event instanceof FocusEvent && event.relatedTarget) {
            var element = event.relatedTarget;
            if (element.classList.contains('dropdown-item')) {
                return;
            }
        }
        this.stopEvent(event);
        if (this.mentionsList) {
            this.mentionsList.show = false;
        }
    };
    MentionsComponent.prototype.isPartMention = function (part) {
        return typeof part.contents !== 'undefined';
    };
    MentionsComponent.prototype.formatMention = function (mention) {
        return this._formatMention(mention.contents);
    };
    MentionsComponent.prototype.displayTransform = function () {
        var _ = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            _[_i] = arguments[_i];
        }
        var replaceIndex = this.markupSearch.groups[this.displayName];
        return _[replaceIndex];
    };
    MentionsComponent.prototype._formatMention = function (contents) {
        var replaceValue = "$" + this.displayName, replaceIndex;
        var result = contents.replace(this.markupSearch.regEx, replaceValue);
        if (result === replaceValue) {
            replaceIndex = "$" + this.markupSearch.groups[this.displayName];
            result = contents.replace(this.markupSearch.regEx, replaceIndex);
        }
        return result;
    };
    MentionsComponent.prototype.stopEvent = function (event) {
        if (event.preventDefault) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();
        }
    };
    MentionsComponent.prototype.setupMentionsList = function (caretPosition) {
        this.startPos = caretPosition;
        this.startNode = window.getSelection().anchorNode;
        this.searchString = '';
        this.stopSearch = false;
        this.showMentionsList();
        this.updateMentionsList();
    };
    MentionsComponent.prototype.handleKeyDown = function (event, caretPosition, characterPressed) {
        var _this = this;
        var keyCode = event.which || event.keyCode;
        if (keyCode === Key.Space) {
            this.startPos = -1;
        }
        else if (keyCode === Key.Backspace && caretPosition > 0) {
            caretPosition--;
            if (caretPosition === this.startPos) {
                this.stopSearch = true;
            }
            this.mentionsList.show = !this.stopSearch;
        }
        else if (this.mentionsList.show) {
            if (keyCode === Key.Tab || keyCode === Key.Enter) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                var value = this._value;
                var start = mapPlainTextIndex(value, this.markupSearch, this.startPos, false, this.displayTransform.bind(this));
                var item = event.item || this.mentionsList.selectedItem;
                var newValue = replacePlaceholders(item, this.markupSearch);
                var newDisplayValue = this._formatMention(newValue);
                caretPosition = this.startPos + newDisplayValue.length;
                var searchString = this.searchString || '';
                value = value.substring(0, start) + newValue + value.substring(start + searchString.length + 1, value.length);
                this.parseLines(value);
                this.startPos = -1;
                this.searchString = '';
                this.stopSearch = true;
                this.mentionsList.show = false;
                this.changeDetector.detectChanges();
                setTimeout(function () {
                    setCaretPosition(_this.textAreaInputElement.nativeElement, caretPosition);
                    _this.onSelect({ target: _this.textAreaInputElement.nativeElement });
                });
                return;
            }
            else if (keyCode === Key.Escape) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                this.stopSearch = true;
                return;
            }
            else if (keyCode === Key.ArrowDown) {
                this.stopEvent(event);
                this.mentionsList.selectNextItem();
                return;
            }
            else if (keyCode === Key.ArrowUp) {
                this.stopEvent(event);
                this.mentionsList.selectPreviousItem();
                return;
            }
            else if (keyCode === Key.Home) {
                this.stopEvent(event);
                this.mentionsList.selectFirstItem();
                return;
            }
            else if (keyCode === Key.End) {
                this.stopEvent(event);
                this.mentionsList.selectLastItem();
                return;
            }
        }
        if (keyCode === Key.ArrowLeft || keyCode === Key.ArrowRight || keyCode === Key.Home || keyCode === Key.End) {
            this.onSelect(event);
            return;
        }
        var mention = this.displayContent.substring(this.startPos + 1, caretPosition);
        if (keyCode !== Key.Backspace) {
            mention += characterPressed;
        }
        this.searchString = mention || '';
        this.updateMentionsList();
    };
    MentionsComponent.prototype.getDisplayValue = function (item) {
        if (typeof item === 'string') {
            return item;
        }
        else if (item[this.displayName] !== undefined) {
            return item[this.displayName];
        }
        return null;
    };
    MentionsComponent.prototype.showMentionsList = function () {
        var _this = this;
        if (!this.mentionsList) {
            var componentFactory = this.componentResolver.resolveComponentFactory(MentionsListComponent);
            var componentRef = this.viewContainer.createComponent(componentFactory);
            this.mentionsList = componentRef.instance;
            this.mentionsList.itemTemplate = this.mentionListTemplate;
            this.mentionsList.displayTransform = this.displayTransform.bind(this);
            this.mentionsList.itemSelected.subscribe(function (item) {
                _this.textAreaInputElement.nativeElement.focus();
                var fakeEvent = { keyCode: Key.Enter, wasSelection: true, item: item };
                _this.onKeyDown(fakeEvent);
            });
            this.mentionsList.displayTransform = this.getDisplayValue.bind(this);
        }
        this.mentionsList.textAreaElement = this.textAreaInputElement.nativeElement;
        this.mentionsList.show = true;
        this.mentionsList.dropUp = this.dropUp;
        this.mentionsList.activeIndex = 0;
        this.mentionsList.position();
        this.ngZone.run(function () { return _this.mentionsList.resetScroll(); });
    };
    MentionsComponent.prototype.updateMentionsList = function () {
        var _this = this;
        if (!this.disableSearch) {
            var items = Array.from(this.mentions);
            if (this.searchString) {
                var searchString = this.searchString.toLowerCase(), searchRegEx_1 = new RegExp(escapeRegExp(searchString), 'i');
                items = items.filter(function (item) {
                    var value = _this.getDisplayValue(item);
                    return value !== null && searchRegEx_1.test(value);
                });
                if (this.maxItems > 0) {
                    items = items.slice(0, this.maxItems);
                }
            }
            this.mentionsList.items = items;
        }
        else {
            this.search.emit(this.searchString);
        }
    };
    MentionsComponent.prototype.parseMarkup = function () {
        if (this.mentionMarkup.length === 0 || this.mentionMarkup[0] !== this.triggerChar) {
            throw new Error("ng-mentions markup pattern must start with the trigger character \"" + this.triggerChar + "\"");
        }
        this.markupSearch = markupToRegExp(this.mentionMarkup);
    };
    MentionsComponent.prototype.parseLines = function (value) {
        var _this = this;
        if (value === void 0) { value = ''; }
        if (value !== this._value) {
            value = value || '';
            var lines = value.split(this.newLine).map(function (line) { return _this.formatMentions(line); });
            var displayContent = lines.map(function (line) { return line.content; }).join('\n');
            if (this.displayContent !== displayContent) {
                this.lines = lines;
                this.displayContent = displayContent;
                this.triggerChange(value);
            }
        }
    };
    MentionsComponent.prototype.formatMentions = function (line) {
        var _this = this;
        var lineObj = { originalContent: line, content: line, parts: [] };
        if (line.length === 0) {
            return lineObj;
        }
        var tags = [], mention;
        var regEx = this.markupSearch.regEx;
        regEx.lastIndex = 0;
        while ((mention = regEx.exec(line)) !== null) {
            tags.push({ indices: { start: mention.index, end: mention.index + mention[0].length } });
        }
        var prevTags = [];
        var content = '';
        tags.slice().sort(function (tagA, tagB) { return tagA.indices.start - tagB.indices.start; }).forEach(function (tag) {
            var expectedLength = tag.indices.end - tag.indices.start;
            var contents = line.slice(tag.indices.start, tag.indices.end);
            if (contents.length === expectedLength) {
                var prevIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                var before = line.slice(prevIndex, tag.indices.start);
                var partMention = { contents: contents, tag: tag };
                lineObj.parts.push(before);
                lineObj.parts.push(partMention);
                prevTags.push(tag);
                content += before + _this.formatMention(partMention);
            }
        });
        var remainingStart = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
        var remaining = line.slice(remainingStart);
        lineObj.parts.push(remaining);
        content += remaining;
        lineObj.content = content;
        return lineObj;
    };
    MentionsComponent.prototype.refreshStyles = function () {
        var _this = this;
        var element = this.textAreaInputElement.nativeElement;
        var computedStyle = getComputedStyle(element);
        this.highlighterStyle = {};
        styleProperties.forEach(function (prop) {
            _this.highlighterStyle[prop] = computedStyle[prop];
        });
        this.changeDetector.detectChanges();
    };
    MentionsComponent.prototype.triggerChange = function (value) {
        this._value = value;
        this.valueChanges.emit(this._value);
        this.changeDetector.detectChanges();
    };
    return MentionsComponent;
}());
/**
 *
 */
export { MentionsComponent };
//# sourceMappingURL=mentions.component.js.map